% =============================================================================
% PixyDist.m CBP 20-07-21
% Auto Fit of y = A/x + B from user data
% Distance in mm, width in pixels
% =============================================================================

pkg load statistics
% Fill-in your data here. The first 
% column is distance, the second is width

A = [
100  114
150  80
200  64
220  58
240  54
260  52
280  48
300  46
320  42
340  40
360  39
];

d = A(:,1);   % distance
w = A(:,2);   % width

% plot raw data
plot(w,d,'o');

% Here is the model dist = m*(1/w) + c
modelfun = @(p,w) (p(1)./w) + p(2);

% YOU MUST ADJUST THE FOLLOWING LINE WITH YOUR ESTIMATED PARAMETER VALUES
% First the slope, then the intercept
startingVals = [15000,-30];
[coefEsts,R,J] = nlinfit(w,d,modelfun,startingVals);
disp(' ');
disp('Estimated Coefficienst'); disp(coefEsts);

% Get smallest and largest x-values 
minX = min(A(:,2));
maxX = max(A(:,2));
xgrid = linspace(minX,maxX,100); 

line(xgrid,modelfun(coefEsts,xgrid), 'Color','r');
set(gca,"fontsize",14);
ylabel('dist')
xlabel('width');

disp(' ' );
disp('95% confidence intervals for parameters');
bci = nlparci(coefEsts,R,J);
disp(bci);

f = modelfun(coefEsts,w);

disp(' ' );
disp('rms error = ');
err = d - f;
err2 = err.^2;
mse = mean(err2);
rmse = sqrt(mse);
disp(rmse);

ssResid = sum(R.^2);
ssTotal = sum((d-mean(d)).^2);
r2 = 1 - (ssResid/ssTotal);
disp('  ');
disp('r2 value = '); disp(r2);